package PersonLib;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;

public class PersonManager {
	private List<Person> personList = null;

	public PersonManager() {
		personList = new ArrayList<Person>();
	}

	public PersonManager(List<Person> personList) {
		this.personList = personList;
	}

	public void loadCSV(String filename) {
		this.personList = PersonImporter.loadCSV(filename);
	}

	public void writeCSV(String filename) {
		PersonImporter.writeCSV(filename, this.personList);
	}

	public List<Person> getList() {
		return this.personList;
	}

	public Person get(int index) {
		return this.personList.get(index);
	}

	public void set(int index, Person p) {
		this.personList.set(index, p);
	}

	public int size() {
		return this.personList.size();
	}

	public boolean isEmpty() {
		return this.personList.isEmpty();
	}

	public void add(Person p) {
		this.personList.add(p);
	}

	public void addAll(List<Person> personList) {
		this.personList.addAll(personList);
	}

	public void print() {
		this.personList.forEach(p -> System.out.println(p));
	}

	// sort by
	public void sort(int index) {
		this.personList.sort(new PersonComparator(index));
	}

	// remove at
	public void removeAt(int index) {
		if (index >= 0 && index < size()) {
			this.personList.remove(index);
		}
	}

	// remove must be done with Iterator or lambda removeIf
	public void removeAllIf(Predicate<Person> func) {
		this.personList.removeIf(func);
	}

	// find if matches
	public List<Person> findAllIf(Predicate<Person> func) {
		return this.personList.stream().filter(func).collect(Collectors.toList());
	}


}
