package PersonTableFrame;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.JTableHeader;

import PersonLib.Person;
import PersonLib.PersonImporter;

public class PersonTableFrame  extends JFrame {
	List<Person> personList = new ArrayList<Person>();
	PersonTableModel model;

	public PersonTableFrame() {
		super("PersonTableFrame");

		personList = PersonImporter.loadCSV("PersonList.csv");
		//personList.forEach((s) -> model.addRow(s));
		//personList.forEach((s) -> System.out.println(s));
		//saveCSV("TestList.csv", personList);

		model = new PersonTableModel(personList);
		JTable table = new JTable(model);

		table.setAutoCreateRowSorter(true); // enable sorting
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setIntercellSpacing(new Dimension(10, 5));
		table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				 ListSelectionModel lsm = (ListSelectionModel)e.getSource();
				 if (!e.getValueIsAdjusting() && !lsm.isSelectionEmpty()) {
					int index = table.getSelectedRow();
					if (index >= 0 && index < personList.size()) { 
						model.removeRow(index); 
						personList.forEach((s) -> System.out.println(s));
					}
				 }
			}
		});
		
		JTableHeader header = table.getTableHeader();
	    header.setUpdateTableInRealTime(true);
	    header.addMouseListener(model.new ColumnListener(table));
	    header.setReorderingAllowed(true);
		
		JScrollPane scrollPane = new JScrollPane(table);
		this.add(scrollPane, BorderLayout.CENTER);

		// menu
		JMenuBar menuBar = new JMenuBar();

		// File Menu, F - Mnemonic
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic(KeyEvent.VK_F);
		menuBar.add(fileMenu);
		
		// File->Exit, E - Mnemonic
		JMenuItem exitMenuItem = new JMenuItem("Exit", KeyEvent.VK_E);
		fileMenu.add(exitMenuItem);
		this.setJMenuBar(menuBar);
		exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent event) {
				PersonImporter.writeCSV("Out3.csv", personList);
				dispose();
			}	
		});

		// Person Menu, P - Mnemonic
		JMenu personMenu = new JMenu("Person");
		personMenu.setMnemonic(KeyEvent.VK_P);
		menuBar.add(personMenu);

		// Person->Add, A - Mnemonic
		JMenuItem addMenuItem = new JMenuItem("Add", KeyEvent.VK_A);
		personMenu.add(addMenuItem);
		this.setJMenuBar(menuBar);
		addMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent event) {
				PersonDialog personDialog = new PersonDialog();
				personDialog.setVisible(true);
				System.out.println("getReturnValue=" + personDialog.getReturnValue());
				if (personDialog.getReturnValue() == PersonDialog.OK) {
					Person p = personDialog.getPerson();
					System.out.println("PersonTableFrame: add " + p);
					model.addRow(p);
					personList.add(p);
					System.out.println("After add:");
					personList.forEach((s) -> System.out.println(s));
					System.out.println("---------------------added");
				}
			}	
		});

		this.setSize(600, 450);
		this.setVisible(true);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

}
