package PersonTableFrame;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import PersonLib.Activity;
import PersonLib.Gender;
import PersonLib.Person;
import PersonLib.PersonComparator;

public class PersonTableModel  extends AbstractTableModel {
	private List<Person> personList;
	private String[] columnNames = Person.columnNames();

	public PersonTableModel(List<Person> pList) {
		this.personList = pList;
	}

	@Override
	public int getColumnCount() {
		return columnNames.length;
	}

	@Override
	public int getRowCount() {
		return this.personList.size();
	}

	@Override
	 public String getColumnName(int column) {
		return columnNames[column];
	 }

	@Override
	 public Class<?> getColumnClass(int columnIndex) {
	 Class<?> type = String.class;
	 switch (columnIndex) {
	 case 0:
	 type = String.class;
	 break;
	 case 1:
	 type = Integer.class;
	 break;
	 case 2:
	 case 3:
	 type = Double.class;
	 break;
	 case 4:
	 type = Gender.class;
	 break;
	 case 5:
	 type = Activity.class;
	 break;
	 }
	 return type;
	 }

	@Override
	 public Object getValueAt(int rowIndex, int columnIndex) {
	 Person p = this.personList.get(rowIndex);
	 Object value = null;
	 switch (columnIndex) {
	 case 0:
	 value = p.getName();
	 break;
	 case 1:
	 value = p.getAge();
	 break;
	 case 2:
	 value = p.getWeight();
	 break;
	 case 3:
	 value = p.getHeight();
	 break;
	 case 4:
	 value = p.getGender();
	 break;
	 case 5:
	 value = p.getActivity();
	 break;
	 }
	 return value;
	 }

	@Override
	 public void setValueAt(Object value, int rowIndex, int columnIndex) {
	 Person p = this.personList.get(rowIndex);
	 switch (columnIndex) {
	 case 0:
	 p.setName((String)value);
	 break;
	 case 1:
	 p.setAge((Integer)value);
	 break;
	 case 2:
	 p.setWeight((Double)value);
	 break;
	 case 3:
	 p.setHeight((Double)value);
	 break;
	 case 4:
	 p.setGender((Gender)value);
	 break;
	 case 5:
	 p.setActivity((Activity)value);
	 break;
	 default:
	 throw new IllegalArgumentException("Invalid column index");
	 }
	fireTableDataChanged();
	 }

	public void addRow(Person p) {
		System.out.println("addRow: " + p + " RowCount=" + getRowCount());
		this.personList.add(p);
		fireTableRowsInserted(this.personList.size() - 1, this.personList.size() - 1);
	}

	public void removeRow(int rowIndex) {
		if (rowIndex >=0 && rowIndex < getRowCount()) {
			System.out.println("PERSONTABLEMODEL REMOVEROW rowIndex=" + rowIndex);
			fireTableRowsDeleted(rowIndex, rowIndex);
			this.personList.remove(rowIndex);
		}
	}

	class ColumnListener extends MouseAdapter {
		private JTable table;
		public ColumnListener(JTable table) {
			this.table = table;
		}

		public void mouseClicked(MouseEvent e) {
			TableColumnModel colModel = table.getColumnModel();
			int columnModelIndex = colModel.getColumnIndexAtX(e.getX());
			int modelIndex = colModel.getColumn(columnModelIndex).getModelIndex();

			if (modelIndex < 0) return; 
			for (int i = 0; i < getColumnCount(); i++) { 
				TableColumn column = colModel.getColumn(i); 
				column.setHeaderValue(getColumnName(column.getModelIndex())); 
			} 
			table.getTableHeader().repaint(); 
			personList.sort(new PersonComparator(columnModelIndex)); 
			table.tableChanged(new TableModelEvent(PersonTableModel.this)); 
			table.repaint(); 
		} 
	} 
}
